/**
 * SERVICE: LoginService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'LoginService', LoginService );

  function LoginService( $q, $state, $timeout, $window, Restangular, MulticommoService, SettingsService, VERSION ) {
    var auth,

      factory = {
        login: login,
        loginWithCredentials: loginWithCredentials,
        switchCommodity: switchCommodity,
        getAuth: getAuth,
        getOrCreateClearCredentials: getOrCreateClearCredentials,
        saveClearCredentials: saveClearCredentials,
        canSeeVoyageCalculator: canSeeVoyageCalculator
      };

    return factory;


    ////////////

    /**
     * POST request to login a user
     * @param email
     * @param password
     * @param cancelPromise
     * @returns {Promise}
     */
    function login( email, password ) {
      var authRequest = {
        email: email,
        password: password
      };

      return Restangular
        .all( 'login' )
        .post( authRequest )
        .then( storeAuth );
    }

    /**
     * @returns {Promise}
     */
    function loginWithCredentials() {
      var authRequest = getClearCredentials();

      if ( !authRequest ) { return $q.reject(); }

      return Restangular
        .all( 'login' )
        .post( authRequest )
        .then( storeAuth );
    }

    function switchCommodity( commo ) {
      var authForCommo,
        credentialsForCommo;

      try {
        authForCommo = localStorage.getItem( 'auth' );
        localStorage.removeItem( 'availableUnits' );
        SettingsService.setCommodity( commo );
        localStorage.setItem( 'current_commodity', JSON.stringify( commo ) );

        if ( authForCommo && JSON.parse( authForCommo )[ commo ] ) {
          pageReloadWhenSwitchingCommo();
        } else {
          credentialsForCommo = JSON.parse( localStorage.getItem( 'clear_credentials' ) );
          if ( credentialsForCommo && credentialsForCommo[ commo ] ) {
            login( credentialsForCommo[ commo ].email, credentialsForCommo[ commo ].password ).then( pageReloadWhenSwitchingCommo );
          } else {
            $state.go( 'login' ).then( function() {
              $timeout( function() {
                $window.location.reload();
              }, 0 );
            } );
          }
        }

      } catch ( err ) {
        console.error( err );
        $state.go( 'login', { reload: true } );
      }
    }

    function pageReloadWhenSwitchingCommo() {
      if ( $state.current.name !== 'session.map' ) {
        $state.go( 'session.map' ).then( function() {
          $timeout( function() {
            $window.location.reload();
          }, 0 );
        } );
      } else {
        $window.location.reload();
      }
    }

    /**
     *
     * @returns {auth}
     */
    function getAuth() {
      if ( auth ) {
        return auth;
      }

      auth = cacheAuthFromLocalStorage();
      return auth;
    }

    /**
     * Retrieve credentials from localStorage
     * @returns {*}
     */
    function getOrCreateClearCredentials() {
      var credentials = getClearCredentials();

      if ( credentials ) { return credentials; }

      return {
        email: '',
        password: ''
      };
    }

    /**
     * Store credentials in localStorage
     * @param credentials
     */
    function saveClearCredentials( credentials ) {
      var json,
        obj = {},
        currentCommo = MulticommoService.getCurrentCommodity().name;

      if ( !credentials || !credentials.email || !credentials.password ) {
        return;
      }

      if ( localStorage.getItem( 'clear_credentials', json ) ) {
        obj = JSON.parse( localStorage.getItem( 'clear_credentials', json ) );
      }
      obj[ currentCommo ] = credentials;
      json = JSON.stringify( obj );
      localStorage.setItem( 'clear_credentials', json );
    }

    /**
     * @todo: This should be replaced by a Role Base Access Control architecture (as in Frontend project)
     * @returns {boolean}
     */
    function canSeeVoyageCalculator() {
      var user = auth.user,
        hasFreightCalculatorPermission = _.includes( user.permissions, 'freight_calculator' ),
        hasVoyageCalculatorPermission = _.includes( user.permissions, 'voyage_calculator' );
      return hasFreightCalculatorPermission || hasVoyageCalculatorPermission;
    }

    ///////////
    // Non exposed methods
    //////////

    /**
     *
     * @returns {*}
     */
    function cacheAuthFromLocalStorage() {
      // Get user from local storage
      var json = localStorage.getItem( 'auth' ),
        authExt,
        currentCommo = MulticommoService.getCurrentCommodity().name;

      if ( !json ) { return; }

      try {
        authExt = JSON.parse( json );
      } catch ( err ) {
        // In case of a wrong storage
        return;
      }
      onConnected( authExt[ currentCommo ] );
      return authExt[ currentCommo ];
    }

    /**
     *
     * @param data
     * @returns {*}
     */
    function storeAuth( data ) {
      var json,
        authToSet,
        currentCommo = MulticommoService.getCurrentCommodity().name;

      if ( data.user && data.token ) {
        if ( localStorage.getItem( 'auth' ) ) {
          authToSet = JSON.parse( localStorage.getItem( 'auth' ) );
        } else {
          authToSet = {};
        }
        authToSet[ currentCommo ] = data.plain();
        json = JSON.stringify( authToSet );
        localStorage.setItem( 'auth', json );
        localStorage.setItem( 'current_commodity', JSON.stringify( currentCommo ) );

        auth = authToSet[ currentCommo ];
        onConnected( auth );

        return auth;
      } else {
        return $q.reject();
      }
    }

    function getClearCredentials() {
      var json = localStorage.getItem( 'clear_credentials' ),
        credentials,
        currentCommo = MulticommoService.getCurrentCommodity().name;

      if ( json ) {
        try {
          credentials = JSON.parse( json )[ currentCommo ];
          if ( credentials && credentials.email && credentials.password ) {
            return credentials;
          }
        } catch ( err ) {
          // Nothing
        }
      }

      return;
    }

    /**
     * Once the user is logged in,
     *   - send analytics to Woopra
     * @param auth
     */
    function onConnected( authUser ) {
      var id,
        traits;

      // For teamboard, reload the state, to restart SSE
      if ( $state.current.name === 'session.teamboard' ) {
        $window.location.reload();
      }

      // Register to Segment
      if ( authUser ) {
        id = authUser.user.id;
        traits = {
          email: authUser.user.email,
          mobile: VERSION.number + ' (historical)'
        };
        analytics.identify( id, traits );
        analytics.group( authUser.user.organisation.id, { organisation: authUser.user.organisation.name } );

        Sentry.configureScope( function( scope ) {
          scope.setUser( { id: id, email: authUser.user.email } );
        } );
      }
    }

  }

} )();
