/**
 * SERVICE: TeamboardService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'TeamboardService', TeamboardService );

  function TeamboardService( $log, $q, $rootScope, Restangular, SettingsService ) {

    var BASE = Restangular.withConfig( RestangularConfig ).all( 'messages' ),
      messages,
      promise,
      feed,
      userId,

      factory = {
        connect: connect,
        disconnect: disconnect,
        postMessage: postMessage,
        getMessages: getMessages
      };

    return factory;


    ////////////

    function connect( auth ) {
      var url = SettingsService.getSettings().TEAMBOARD + '/messagesFeed',
        options = {
          headers: {
            Authorization: auth.token
          }
        };

      userId = auth.user.id;

      feed = new EventSource( url, options );
      feed.addEventListener( 'message', receiveMessage );
      feed.addEventListener( 'heartbeat', receiveHeartbeat );

      ////////////

      function receiveMessage( msg ) {
        var msgJson = JSON.parse( msg.data ),
          msgFmt = formatMessage( msgJson );

        $rootScope.$apply( function() {
          if ( messages ) {
            messages.shift();
            messages.push( msgFmt );
          }

          $rootScope.$emit( 'teamboard:newmessage', msgFmt );

        } );
      }

      function receiveHeartbeat() {
        switch ( feed.readyState ) {
          case EventSource.CONNECTING:
            $log.info( 'SSE Connecting...' );
            break;
          case EventSource.OPEN:
            $log.info( 'SSE Open.' );
            break;
          case EventSource.CLOSED:
            $log.info( 'SSE Closed.' );
            break;
          default:
            // this never happens
            break;
        }
      }
    }

    function disconnect() {
      if ( feed ) {
        feed.close();
        feed = null;
      }
      promise = null;
    }

    function postMessage( message ) {
      var messageReq = {
        text: message
      };

      return BASE.post( messageReq );
    }

    function getMessages() {
      return $q.when( messages || promise || fetchMessages() );


      ////////////

      function fetchMessages() {
        var messagesReq = {
          limit: 60,
          offset: 0
        };

        promise = BASE.getList( messagesReq )
          .then( formatAndAddMessages )
          .catch( cleanPromise );

        return promise;


        ////////////

        function formatAndAddMessages( data ) {
          var i,
            length = data.length,
            msg;

          if ( !messages ) {
            messages = [];
          }

          for ( i = 0; i < length; i++ ) {
            msg = data[ i ].plain();
            formatMessage( msg );
            messages.unshift( msg );
          }

          return messages;
        }

        function cleanPromise() {
          promise = null;
        }
      }
    }

    function RestangularConfig( RestangularConfigurer ) {
      RestangularConfigurer.setBaseUrl( SettingsService.getSettings().TEAMBOARD );
    }

    function formatMessage( newMessage ) {
      // From (formatted)
      newMessage.fromFormatted = extractFormatted( newMessage.from );

      // Date
      newMessage.dateHuman = global.datesUtils.humanize( newMessage.date );

      // Type
      if ( !newMessage.organisationId ) {
        newMessage.clazz = 'message-admin';
      } else if ( newMessage.userId === userId ) {
        newMessage.clazz = 'message-me';
      } else {
        newMessage.clazz = 'message-other';
      }

      return newMessage;
    }

    function extractFormatted( from ) {
      var patt = new RegExp( '^\\S*\\s?\\w?' ),
        res = patt.exec( from );

      if ( res && res.length > 0 ) {
        return res[ 0 ].slice( 0, 10 );
      } else {
        return null;
      }
    }
  }

} )( this );
