/**
 * SERVICE: VesselService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'VesselService', VesselService );

  function VesselService( $q, Restangular, MapApiTransformsService, MapCacheService, ActivityService ) {

    var factory = {
      getById: getById,
      loadPositions: loadPositions
    };

    return factory;


    ////////////

    function getById( id, cancelPromise ) {
      var promiseBaseFromCache,
        promiseDetailsFromWeb,
        promisePC;

      promiseBaseFromCache = MapCacheService.getVesselById( id );
      promiseDetailsFromWeb = Restangular.one( 'vessels', id );

      if ( cancelPromise ) {
        promiseDetailsFromWeb = promiseDetailsFromWeb.withHttpConfig( {
          timeout: cancelPromise
        } );
      }

      promiseDetailsFromWeb = promiseDetailsFromWeb
        .get()
        .then( extractPlain );

      promisePC = ActivityService.load( id, 0, cancelPromise, 'vessels' )
        .then( function( portCalls ) {
          return {
            portCalls: portCalls
          };
        } );

      return $q.all( [
        promiseDetailsFromWeb,
        // Base is not the first because we merge the 1st with other and base is cached
        promiseBaseFromCache,
        promisePC
      ] )
        .then( mergePromises )
        .then( updateStatusText )
        .then( processData );


      ///////////////

      function extractPlain( data ) {
        return data.plain();
      }

      function mergePromises( promises ) {

        var destination,
          i,
          promise;

        if ( !promises || promises.length <= 0 ) {
          return;
        }

        destination = promises[ 0 ];
        for ( i = 0; i < promises.length; ++i ) {
          promise = promises[ i ];
          angular.extend( destination, promise );
        }

        return destination;
      }

      function updateStatusText( vessel ) {
        vessel.$statusText = global.textTranslateHelper.translateStatus( vessel.status );

        return vessel;
      }

      function processData( data ) {
        // Update type
        updateType( data );

        data.statusClass = global.clazzHelper.getStatusClazz( data.status, data.resourceType );
        data.carrierType = carrierType( data.classification );

        // Specific - Portcalls / Voyage
        updateNextDestination( data );

        return data;


        ////////////

        function updateType( item ) {
          if ( !item.type ) {
            if ( item.resourceType === 'player' ) {
              item.type = 'Player';
            } else if ( item.resourceType === 'zone' ) {
              item.type = 'Zone';
            } else if ( item.resourceType === 'vessel' ) {
              item.type = 'Vessel';
            } else if ( item.resourceType === 'installation' ) {
              //TODO: refactor preprocessItemDetails differently to use for vessel/installation and others items
              //nothing, already on item;
            } else {
              item.type = 'Unknown';
            }
          }
        }


        function carrierType( classification ) {
          if ( !classification || !classification.carrierType ) {
            return;
          }

          if ( classification.carrierType === 'Ocean' ) {
            return;
          }

          return classification.carrierType;
        }

        function updateNextDestination( item ) {
          var i,
            portCalls = item.portCalls,
            len = portCalls.length,
            portCall,
            nextDestinationNotFound = true;

          if ( !portCalls ) {
            return;
          }

          for ( i = 0; i < len; i++ ) {

            portCall = portCalls[ i ];

            //set nextDestination (not forecasted)
            if ( nextDestinationNotFound && portCall.future && !portCall.forecasted && !portCall.cancelled ) {
              item.nextDestination = portCall;
            } else if ( portCall.arrival ) {
              nextDestinationNotFound = false;
            }

          }

        }
      }
    }

    function loadPositions( vessel ) {

      var before,
        req,
        headers,
        after;

      before = moment.utc();

      req = {
        before: before.format()
      };

      after = findNextArrival( vessel.portCalls, 1 );

      if ( after ) {
        // For inactive vessel
        req.after = after.format();
      }

      headers = { 'X-Version': 1.0 };

      return Restangular
        .one( 'vessels', vessel.id )
        .one( 'positions' )
        .get( req, headers )
        .then( updatePositions )
        .then( sortAndRemoveDuplicates );


      ////////////

      function updatePositions( positions ) {
        var i = 0,
          len = positions.length;

        for ( i; i < len; i++ ) {
          update( positions[ i ] );
        }

        return positions;

        ////////////

        function update( position ) {
          position.$geo = MapApiTransformsService.formatPosition( position.geo );
          position.$receivedTime = moment( position.receivedTime );
          position.$receivedTimeHuman = global.datesUtils.humanize( position.$receivedTime );
        }
      }

      function sortAndRemoveDuplicates( positions ) {
        if ( positions ) {
          return _( positions )
            .sortBy( function( pos ) {
              return -pos.$receivedTime.unix();
            } )
            .uniq( true, function( pos ) {
              return pos.id;
            } )
            .value();
        }
      }

      function findNextArrival( portCalls, maxPortCalls ) {
        if ( !portCalls ) {
          return;
        }

        return _( portCalls )
          .filter( function( portCall ) {
            return !portCall.future && !portCall.forecasted && !portCall.cancelled && portCall.arrival;
          } )
          .map( function( portCall ) {
            return portCall.arrival;
          } )
          .filter( function( item, index ) {
            return index === ( maxPortCalls - 1 );
          } )
          .first();
      }

    }
  }

} )( this );
