/**
 * DIRECTIVE: pcKnownVessel
 * @author Jean Maynier <jmaynier@kpler.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'VoyageCalculatorService', voyageCalculatorService );

  function voyageCalculatorService( Restangular, MapApiTransformsService, MapCacheService, SearchService ) {
    var resourceVC = Restangular.all( 'voyage_calculator' ),

      factory = {
        calculate: calculate,
        clear: clear,
        backup: { viaPoints: [] },
        result: null,
        origin: null,
        destination: null,
        getOriginSearchResults: getOriginSearchResults,
        getViaPoints: getViaPoints,
        getDestinationSearchResults: getDestinationSearchResults
      };

    return factory;


    ////////////

    function hasAPosition( waypoint ) {
      var getById = {
        vessel: MapCacheService.getCachedVesselById,
        installation: MapCacheService.getCachedInstallationById,
        zone: function() { return { $geo: true }; }
      };
      var cached = getById[ waypoint.resourceType ]( waypoint.id );
      return cached && !!cached.$geo;
    }

    function getOriginSearchResults( q ) {
      return SearchService.searchVC( q, true, [ 'vessel', 'installation', 'zone' ] )
        .then( function( data ) {
          return _.chain( data )
            .flatten()
            .filter( function( x ) { return ( x.resourceType !== 'zone' && hasAPosition( x ) ) || x.resourceType === 'zone'; } )
            .value();
        } );
    }

    function getViaPoints( q ) {
      return SearchService.searchVC( q, true, [ 'zone' ] )
        .then( function( data ) {
          return _.chain( data )
            .flatten()
            .value();
        } );
    }

    function getDestinationSearchResults( q ) {
      return SearchService.searchVC( q, true, [ 'installation', 'zone' ] )
        .then( function( data ) {
          return _.chain( data )
            .flatten()
            .filter( function( x ) { return ( x.resourceType !== 'zone' && hasAPosition( x ) ) || x.resourceType === 'zone'; } )
            .value();
        } );
    }


    function clear() {
      factory.result = null;
      factory.origin = null;
      factory.destination = null;
    }

    function calculate( origin, destination, speed, viaPoints ) {
      var viaZoneIds = _.map( viaPoints, function( point ) {
        if ( point && point.res ) {
          return point.res.id;
        }
      } );

      return resourceVC
        .one( origin.resourceType, origin.id )
        .one( destination.resourceType, destination.id )
        .post( null, { viaZoneIds: viaZoneIds } )
        .then( changeLatLng )
        .then( updateDurationHumanize )
        .then( registerVoyage );


      ////////////

      function changeLatLng( result ) {
        if ( result.waypoints ) {
          result.waypoints = _.map( result.waypoints, function( waypoint ) {
            return MapApiTransformsService.formatPosition( waypoint );
          } );
        }

        return result;
      }

      /**
       * @param result
       * @returns {*}
       * @todo: Check days / hours type to use strict '===' condition
       */
      function updateDurationHumanize( result ) {
        var duration,
          days,
          hours,
          daysHumanize,
          hoursHumanize;

        if ( speed > 0 ) {

          duration = moment.duration( result.distance / speed, 'hours' );

          result.duration = duration;

          days = Math.round( duration.asDays() );
          daysHumanize =
            days === 0 ? '' :
              days === 1 ? '1 day' :
              days + ' days';

          hours = duration.hours();
          hoursHumanize =
            days > 3 ? '' :
              hours === 1 ? '1 hour' :
              hours + ' hours';

          result.duration = daysHumanize + ' ' + hoursHumanize;
        } else {
          result.duration = '-';
        }
        return result;
      }

      function registerVoyage( result ) {
        factory.result = result;
        factory.origin = origin;
        factory.destination = destination;
        return result;
      }

    }
  }

} )();
