/**
 * * DIRECTIVE: contract
 * @author Jin
 */

( function( global ) {
  'use strict';

  angular
    .module( 'lng' )
    .directive( 'contract', contract );

  function contract() {
    var directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        contract: '='
      },
      templateUrl: 'components/contracts/contract/contract.html',
      controller: contractController,
      controllerAs: 'vm'
    };

    return directive;

    ////////////

    function contractController( $sce, $filter, LoginService ) {

      var vm = this,
        filterTranslate = $filter( 'translate' ),
        defaultDate = global.datesUtils.momentFullDate,
        $contract = this.contract;

      this.hasContractAggregationPermission = LoginService.getAuth().user.permissions.indexOf( 'contract_aggregation' ) > -1;

      updateContract();

      /////////

      function updateContract() {

        // The gaugeConfig object will be passed to the gauge directive
        // to configure it
        var gaugeConfig = vm.gaugeConfig = {},
          fulfillment,
          precision,
          filterNumber;

        vm.startHuman = $contract.startDate ? defaultDate( $contract.startDate ) : 'N/A';
        vm.endHuman = $contract.endDate ? defaultDate( $contract.endDate ) : 'N/A';

        // Update time class
        vm.clazz = global.clazzHelper.getTimeInfoClazz( $contract.startDate, $contract.endDate );

        // Upate tag class
        switch ( $contract.type ) {
          case 'Spot':
          case 'Stripe':
          case 'Tender':
            vm.tagClazz = [ 'short-term', $contract.type.toLowerCase() ];
            break;
          default:
            vm.tagClazz = [ 'long-term', $contract.type.toLowerCase() ];
            break;
        }

        // Update Contract Type
        switch ( $contract.type ) {
          case 'SPA': {
            vm.isSpa = true;
            gaugeConfig.maxValue = $contract.capacity;
            gaugeConfig.baseUnit = 'mtpa';
            break;
          }

          case 'LTA':
          case 'TUA': {
            vm.isTua = true;

            if ( $contract.installationsSales && $contract.installationsSales.length > 0 ) {
              vm.tuaInstallations = $contract.installationsSales;
            } else {
              vm.tuaInstallations = $contract.installationsDelivery;
            }

            break;
          }

          case 'Tender': {
            vm.isTender = true;
            gaugeConfig.maxValue = $contract.slot;
            gaugeConfig.baseUnit = 'slot';
            gaugeConfig.pluralizeUnit = true;
            break;
          }

          case 'Strip': {
            vm.isStrip = true;
            break;
          }
        }

        // Get fulfillment data from the contract object
        fulfillment = getFulfillmentData();

        // Add fulfillment data to the gaugeConfig object
        if ( fulfillment.available && vm.hasContractAggregationPermission ) {

          gaugeConfig.display = true;
          // Add 1 segment for the confirmed part
          // and 1 segment for the potential part
          gaugeConfig.segments = [
            {
              value: fulfillment.confirmedValue,
              className: 'bgConfirmed'
            }, {
              value: fulfillment.potentialValue,
              className: 'bgPotential'
            }
          ];

          precision = vm.isTender ? 0 : 2;
          filterNumber = function( n ) {
            return $filter( 'number' )( n, precision );
          };

          gaugeConfig.getBarTooltipText = function() {
            // @todo use a template file instead of this
            return $sce.trustAsHtml(
              ( fulfillment.year
                ? fulfillment.year + '<br>'
                : '' )
              + ( fulfillment.confirmedValue
                ? filterTranslate( 'COMPONENTS.CONTRACTS.CONTRACT.CONFIRMED' ) + ': ' + filterNumber( fulfillment.confirmedValue ) + ' / ' + ( this.maxValue ? filterNumber( this.maxValue ) : '?' ) + ' ' + this.baseUnit + ( this.pluralizeUnit && fulfillment.confirmedValue > 1 ? 's' : '' )
                : '' )
              + ( fulfillment.confirmedValue && fulfillment.potentialValue
                ? '<br>'
                : '' )
              + ( fulfillment.potentialValue
                ? filterTranslate( 'COMPONENTS.CONTRACTS.CONTRACT.POTENTIAL' ) + ': ' + filterNumber( fulfillment.potentialValue ) + ' / ' + ( this.maxValue ? filterNumber( this.maxValue ) : '?' ) + ' ' + this.baseUnit + ( this.pluralizeUnit && fulfillment.potentialValue > 1 ? 's' : '' )
                : '' )
            );
          };
        } else {

          gaugeConfig.display = false;

        }

        ////////////

        /**
         * Get volume or number of slot(s) of a contract by year
         * This method currently returns only the latest year available for non-Tenders and
         *
         * @returns {{available: boolean, unit: string, value: int}}
         */
        function getFulfillmentData() {

          var confirmed,
            potential,
            perYear,
            currentYear,
            fulfillmentThisYear,
            // the fulfillment object for the current year
            result = {
              available: true,
              unit: '',
              confirmedValue: 0,
              potentialValue: 0,
              year: undefined
            };

          // Check if fulfillment object is available in the contract
          if ( !$contract.fulfillment ) {

            result.available = false;

          } else if ( $contract.type === 'Tender' ) {

            result.unit = 'slot';
            confirmed = $contract.fulfillment.confirmed;
            potential = $contract.fulfillment.potential;

            if ( confirmed ) {
              result.confirmedValue = confirmed.nSlot || 0;
            } else if ( potential ) {
              result.potentialValue = potential.nSlot || 0;
            } else {
              result.available = false;
            }

          } else {

            result.unit = 'mtpa';
            perYear = $contract.fulfillment.perYear;

            if ( perYear ) {

              // Find current year fulfillment
              // Get current year and convert it to a timestamp
              result.year = moment().format( 'YYYY' ).toString();
              currentYear = new Date( result.year ).getTime();

              // Use this timestamp to find current fulfillment
              fulfillmentThisYear = _.find( perYear, function( oneYearFulfillment ) {
                return oneYearFulfillment.date === currentYear;
              } );

              if ( fulfillmentThisYear ) {
                confirmed = fulfillmentThisYear.confirmed;
                potential = fulfillmentThisYear.potential;

                // @todo: Remove the division by 1 million once API is corrected.
                // Unit inconsistency. It currently returns 'mtpa' for '$contract.capacity' key but 'tpa'
                // for 'desTon/fobTon' keys inside '$contract.perYear' Array
                // @todo react to changes in user unit selection when that setting becomes available
                // There is a 'ton' field but also a 'volume' field
                if ( confirmed ) {
                  result.confirmedValue = ( confirmed.ton || 0 ) / 1000000;
                } else if ( potential ) {
                  result.potentialValue = ( potential.ton || 0 ) / 1000000;
                } else {
                  result.available = false;
                }
              } else {
                result.available = false;
              }
            } else {
              result.available = false;
            }

          }
          return result;
        }

      }
    }
  }

} )( this );
