/**
 * DIRECTIVE: gauge
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .component( 'gauge', {
      bindings: {
        /**
         * gaugeConfig:: {
                 *   segments: [Segment],
                 *   maxValue: (number),       // optional (if absent or invalid, sum of segment values is used instead)
                 *   baseUnit: (string),       // optional (default: '')
                 *   pluralizeUnit: (boolean), // optional (default: false)
                 *   getSegmentText: (function (segment) { return String; }),        // optional (default: undefined)
                 * }
         *
         * where
         *
         * Segment:: {
                 *   // User-defined attributes
                 *   value: (number),
                 *   description: (string),
                 *
                 *   // Computed attributes
                 *   unit: (string),
                 *   percent: (int),
                 * }
         *
         * Note: The user should only specify the 'value' and 'description' of each segment
         */
        gaugeConfig: '<',

        // Possible values: true (display everywhere), false (never display),
        // 'segment' (just on the segment)
        displayMaxValue: '<',

        // Possible values: true (display everywhere), false (never display),
        // 'segment' (just on the segment)
        displayUnit: '<'
      },
      templateUrl: 'components/gauge/gauge.html',
      controller: GaugeController
    } );

  /* @ngInject */
  function GaugeController() {

    var vm = this,
      gaugeConfig = this.gaugeConfig,
      baseUnit,
      mustComputeSegmentTexts,
      sum;

    ////////////
    vm.segments = _.filter( gaugeConfig.segments, function( segment ) {
      return segment && _.isFinite( segment.value ) && segment.value !== 0;
    } );

    if ( !vm.segments.length ) {
      return;
    }

    baseUnit = gaugeConfig.baseUnit || 'k';
    mustComputeSegmentTexts = gaugeConfig.getSegmentText === undefined;
    sum = 0;

    vm.clazz = vm.segments[ 0 ].value >= 0 ? 'positive-value' : 'negative-value';
    vm.displayMaxValue = vm.displayMaxValue || false;
    vm.getSegmentText = gaugeConfig.getSegmentText || function( segment ) {
      return segment.text;
    };

    // If the maxValue is specified and valid, use that. Otherwise compute from the sum
    // of the segment values
    if ( _.isFinite( gaugeConfig.maxValue ) ) {
      vm.computedMaxValue = gaugeConfig.maxValue;
      vm.unspecifiedMaxValue = false;
    } else {
      vm.computedMaxValue = vm.segments.reduce( function( segmentSum, segment ) {
        return segmentSum + segment.value;
      }, 0 );
      vm.unspecifiedMaxValue = true;
    }

    vm.segments.forEach( computeSegment );

    // After the % computation, make sure the sum of percents does not exceed 100. If it does,
    // multiply all percentages by 100/(sum of percents).
    if ( sum > 100 ) {
      vm.segments.forEach( function( segment ) {
        segment.percent *= 100 / sum;
      } );
    }

    ////////////

    function computeSegment( segment ) {
      var roundedValue,
        roundedMaxValue,
        nonZeroSegmentsCount,
        atLeastOneDescription;

      // We're going to count the number of segments with non-zero values
      nonZeroSegmentsCount = 0;
      // and we're going to try to know if at least one segment has a description string
      atLeastOneDescription = false;

      if ( segment.value === undefined ) {
        return;
      }

      if ( segment.value ) {
        ++nonZeroSegmentsCount;
        atLeastOneDescription = atLeastOneDescription || segment.description;
      }

      // Compute segment texts
      if ( mustComputeSegmentTexts ) {
        segment.unit = baseUnit;

        if ( gaugeConfig.pluralizeUnit && ( segment.value > 1 || segment.value < -1 ) ) {
          segment.unit += 's';
        }

        // Compute the text that is going to appear on the segment
        // @todo allow the user of this directive to specify a floating point precision instead of hardcoding the 100?
        roundedValue = Math.round( 100 * segment.value ) / 100;
        roundedMaxValue = Math.round( 100 * vm.computedMaxValue ) / 100;

        if ( mustComputeSegmentTexts ) {
          segment.text = (
            roundedValue
            + ( ( vm.displayMaxValue === true || vm.displayMaxValue === 'segment' ) ? ' / ' + roundedMaxValue : '' )
            + ( ( vm.displayUnit === true || vm.displayUnit === 'segment' ) ? ' ' + segment.unit : '' )
          );
        }

      }

      // Set the "percent" field of the sgegment object which is going to be used to determine
      // the segment's width.
      segment.percent = ( 100 * Math.abs( segment.value ) ) / vm.computedMaxValue;
      sum += segment.percent;
    }

  }

} )();
