/**
 * SERVICE: LngMapCacheService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'LngMapCacheService', LngMapCacheService );

  function LngMapCacheService( $ionicLoading, $q, $state, LngMapService, MapCacheService, VesselService ) {
    var container,
      _mapPromise,
      dataPromise,
      selectedMarker,
      selectedTrack,
      cancelTrack = false,

      factory = {
        getContainer: getContainer,
        addVoyageTrack: addVoyageTrack,
        selectVessel: selectVessel,
        unselectVessel: unselectVessel,
        refresh: refresh,
      };

    return factory;


    ////////////

    function getContainer() {
      if ( !container ) {
        container = createContainer();
      }

      return container;
    }

    function createContainer() {

      var viewPortContainer = angular.element( '<div class="lng-map"></div>' ),
        viewPort = angular.element( '<div class="lng-map-viewport" data-tap-disabled="true"></div>' );

      $ionicLoading.show( {
        content: 'Loading data',
        animation: 'fade-in'
      } );

      viewPortContainer.append( viewPort );

      _mapPromise = LngMapService.createMap( viewPort[ 0 ], 52, 0 );
      dataPromise = MapCacheService.load();

      $q.all( [ _mapPromise, dataPromise ] )
        .then( function( results ) {
          var map = results[ 0 ],
            data = results[ 1 ],
            i,
            len,
            vessels = data[0],
            installations = data[1];

        // avoid display issues by clearing items data when the map is recreated
          len = vessels.length;
          for ( i = 0; i < len; i++ ) {
            delete vessels[ i ].$mapAttrs;
          }

          len = installations.length;
          for ( i = 0; i < len; i++ ) {
            delete installations[ i ].$mapAttrs;
            delete installations[ i ].$marker;
          }

        // add markers
          LngMapService.addVessels( map, vessels );
          LngMapService.addInstallations( map, installations );
        } )
        .finally( function() { $ionicLoading.hide(); } );

      LngMapService.setAutoZoom( _mapPromise );
      // add listeners
      LngMapService.addClickListener( _mapPromise, onItemSelected );

      return viewPortContainer;


      ////////////

      function onItemSelected( marker ) {
        // FIXME: click event fired even with ev.stopPropagation()
        // so canceling manually the redirection when on vessel page
        if ( $state.current.name === 'session.vessel' ) { return; }

        if ( marker && marker.$itemType ) {
          switch ( marker.$itemType ) {
            case 'installation': {
              $state.go( 'session.installation', { itemId: marker.$itemId } );
              break;
            }
            case 'vessel': {
              $state.go( 'session.map.vessel', {
                id: marker.$itemId,
                zoom: false
              } );
              break;
            }
            default: {
              throw new Error( 'Not implemented' );
            }
          }
        } else if ( $state.current.name !== 'session.map.voyage' ) {
          $state.go( 'session.map' );
        }
      }
    }

    function addVoyageTrack( waypoints ) {
      var promise = LngMapService.addVoyageTrack( _mapPromise, waypoints );
      return _.partial( removeVoyageTrack, promise );

      ////////////

      function removeVoyageTrack( resolve ) {
        return resolve.then( function( remove ) { remove(); } );
      }
    }

    function selectVessel( vessel, zoom ) {

      _mapPromise.then( function( map ) {
        selectedMarker = vessel.$mapAttrs.$marker;

        if ( selectedMarker ) {
          global.markerVesselHelper.select( map, selectedMarker );

          cancelTrack = false;

          // Add track
          VesselService.loadPositions( vessel )
            .then( function( positions ) {

              if ( !cancelTrack ) {
                LngMapService.addTrackToVessel( map, vessel, positions, zoom )
                .then( function( track ) {
                  if ( selectedTrack ) { selectedTrack.remove(); }
                  selectedTrack = track;
                } );
              }
            } );
        }
      } );
    }

    function unselectVessel() {
      _mapPromise.then( function( map ) {
        if ( selectedMarker ) {
          global.markerVesselHelper.unselect( map, selectedMarker );
          selectedMarker = undefined;
        }

        if ( selectedTrack ) {
          selectedTrack.remove();
          selectedTrack = undefined;
          cancelTrack = false;
        } else {
          cancelTrack = true;
        }
      } );
    }

    function refresh() {
      return $q.all( [ _mapPromise, dataPromise ] )
        .then( function( data ) {
          var map = data[ 0 ];
          resizeMap( map );
        } );
    }

    function resizeMap( map ) {
      var listener = google.maps.event.addListener( map, 'resize', function() {
        google.maps.event.removeListener( listener );
      } );

      google.maps.event.trigger( map, 'resize' );
    }
  }

} )( this );
