/**
 * Map Helper
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';

  var defaultMapStyles;

  global.mapUtils = ( global.module || {} ).exports = {
    createMap: createMap,
    zoomToBounds: zoomToBounds
  };

  ////////////

  defaultMapStyles = [
    {
      featureType: 'water',
      elementType: 'all',
      stylers: [
        { color: '#0e171d' },
        { visibility: 'on' }
      ]
    },
    {
      featureType: 'water',
      elementType: 'labels.text.fill',
      stylers: [
        { color: '#33404a' }
      ]
    },
    {
      featureType: 'water',
      elementType: 'labels.text.stroke',
      stylers: [
        { visibility: 'off' }
      ]
    },
    {
      featureType: 'administrative',
      elementType: 'all',
      stylers: [
        { visibility: 'off' }
      ]
    },
    {
      featureType: 'administrative.country',
      elementType: 'geometry.stroke',
      stylers: [
        { color: '#0e171d' },
        { visibility: 'on' },
        { weight: 0.4 }
      ]
    },
    {
      featureType: 'poi',
      elementType: 'all',
      stylers: [
        { visibility: 'off' }
      ]
    },
    {
      featureType: 'road',
      elementType: 'all',
      stylers: [
        { visibility: 'off' }
      ]
    },
    {
      featureType: 'transit',
      elementType: 'all',
      stylers: [
        { visibility: 'off' }
      ]
    },
    {
      featureType: 'landscape',
      elementType: 'all',
      stylers: [
        { color: '#33404a' }
      ]
    },
    {
      featureType: 'landscape',
      elementType: 'labels',
      stylers: [
        { visibility: 'off' }
      ]
    }
  ];


  ////////////

  /**
   *
   * @param gm
   * @param domElement
   * @param center
   * @returns {boolean|*}
   */
  function createMap( gm, domElement, center ) {

    var mapOptions,
      map,
      style,
      lastValidBounds,
      allowedBounds,
      levelState;

    // Map options
    //
    mapOptions = {
      disableEvents: 'all',
      center: center,
      minZoom: 2,
      zoom: 3,
      scaleControl: true,
      mapTypeControl: false,
      streetViewControl: false,
      fullscreenControl: false,
      zoomControl: false,
      rotateControl: false,
      mapTypeId: 'default'
    };

    map = new gm.Map(
      domElement,
      mapOptions );

    // Style the map
    //
    style = new gm.StyledMapType( defaultMapStyles );
    map.mapTypes.set( 'default', style );

    // Configure limit boundaries
    //
    // to avoid the user going out of the map
    // http://bit.ly/1IqG7Vx
    // See preventOutOfBounds() method in this file
    lastValidBounds = map.getBounds();
    allowedBounds = new google.maps.LatLngBounds(
      new google.maps.LatLng( -85, -180 ),
      new google.maps.LatLng( 85, 180 )
    );

    // Level state helper
    //
    // We want to update the size of vessels icons when zoom changes but only on specific levels of zoom change
    //
    // This update should not be performed with 'zoom_changed' event as it leads to unecessary calculations and poor performances
    // Thus, icon size updates must be performed with 'idle' event.
    //
    // The 'idle' event fires at the end of each user interaction (panning, draging, zoom)
    // The following state variable allows to control when updates must be processed and avoid to perform unecessary calculations
    //
    levelState = ( map.getZoom() > 3 ) ? ( map.getZoom() > 7 ? 3 : 2 ) : 1;

    // Create spiderfier overlay
    // @todo: Check if this spiderfier does not impact perfs. To remove if so.
    map.$oms = createSpiderfierOverlay( map );

    // Configure map events
    //
    // Zoom changes :
    //  - Update map tiles type
    //  - Prevent Out of bounds (useful if zooming out at the limit of the map)
    gm.event.addListener( map, 'zoom_changed', function() {

      zoomChangedListener();
      preventOutOfBounds();

    } );
    //
    // Idle (user interaction stops) :
    //  - Prevent Out of bounds (useful when panning out of the limit of the map)
    //  - Viewport optimization : Display only visible markers
    //  - Update the size of each icon (only if zoom has changed)
    gm.event.addListener( map, 'idle', function() {

      var markers = map.$oms.getMarkers(),
        zoom = map.getZoom(),
        newLevel = ( zoom > 3 ) ? ( zoom > 7 ? 3 : 2 ) : 1;

      if ( newLevel !== levelState ) {
        levelState = newLevel;
        updateMarkersSize( markers, zoom );
      }

      displayMarkersInBounds( markers );
      preventOutOfBounds();

    } );

    return map;

    ////////////

    /**
     *
     */
    function zoomChangedListener() {
      var zoomLevel;

      if ( map.mapTypeControl ) {
        return;
      }

      zoomLevel = map.getZoom();
      if ( zoomLevel > 8 ) {
        map.setMapTypeId( gm.MapTypeId.SATELLITE );
      } else {
        map.setMapTypeId( 'default' );
      }
    }

    /**
     *
     * @param markers
     */
    function displayMarkersInBounds( markers ) {
      var mapBounds = map.getBounds(),
        marker,
        i = 0,
        len = markers.length,
        inBounds,
        visible;

      if ( mapBounds ) {

        // Then add inBounds markers to the map and remove others
        for ( i; i < len; i++ ) {

          marker = markers[ i ];

          inBounds = mapBounds.contains( marker.getPosition() );
          visible = !!marker.getMap();

          // show only if visible on viewport
          if ( inBounds && !visible && marker.$hidden ) {
            marker.setMap( map );
            marker.$hidden = false;
          } else if ( !inBounds && visible ) {
            marker.setMap( null );
            marker.$hidden = true;
          }

        }
      }

    }

    /**
     *
     * @param currentMap
     * @returns {*}
     */
    function createSpiderfierOverlay( currentMap ) {
      var oms;

      global.markerSpiderifyHelper.init( google );

      oms = new OverlappingMarkerSpiderfier( currentMap, {
        nearbyDistance: 10,
        spiderfiedZIndex: gm.Marker.MAX_ZINDEX + 10,
        circleSpiralSwitchover: 7
      } );

      oms.legColors.usual.default = '#fff';
      oms.legColors.highlighted.default = '#f00';

      oms.addListener( 'spiderfy', redrawSpiderfierOverlay );
      oms.addListener( 'unspiderfy', redrawSpiderfierOverlay );

      return oms;

      ////////////

      /**
       *
       * @param markers
       */
      function redrawSpiderfierOverlay( markers ) {

        var i = 0,
          len = markers.length,
          mk;

        for ( i; i < len; i++ ) {
          mk = markers[ i ];
          if ( mk.$label ) {
            mk.$label.set( 'position', mk.position );
          }
        }

      }
    }

    /**
     * @description: Prevent the map from going OOB when a drag ends and also when idle
     * http://bit.ly/1IqG7Vx
     */
    function preventOutOfBounds() {
      var currentBounds = map.getBounds();
      if ( allowedBounds.contains( currentBounds.getSouthWest() ) && allowedBounds.contains( currentBounds.getNorthEast() ) ) {
        // still within valid bounds, so save the last valid position
        lastValidBounds = map.getBounds();
      } else {
        map.panToBounds( lastValidBounds );
      }
    }

    /**
     * Update markers icon size
     * Placing this methods here in spite of marker-{vessel, installation}.helpers.js
     * allows to avoid unnecessary duplicated calculations
     *
     * Currently, marker size modification according to zoom is only implemented for Vessels markers
     *
     * @param {array} markers
     * @param zoom - Current zoom level of the gmap
     *
     */
    function updateMarkersSize( markers, zoom ) {

      var i = 0,
        len = markers.length,
        mk,

        // Set up vessel icons
        iconVesselsBase = global.markerIconHelper.iconConfigShared( zoom, 'vessel' ),
        iconSelected = global.markerIconHelper.iconConfigSelected(),

        // Set up installation icons
        iconInstallationsBase = global.markerIconHelper.iconConfigShared( zoom, 'installation' );

      for ( i; i < len; i++ ) {

        mk = markers[ i ];

        if ( mk.$itemType === 'vessel' ) {
          if ( mk.$selected ) {
            mk.setIcon( global.markerIconHelper.iconConfigSpecific( mk, iconSelected ) );
          } else {
            mk.setIcon( global.markerIconHelper.iconConfigSpecific( mk, iconVesselsBase ) );
          }
        } else {
          mk.setIcon( global.markerIconHelper.iconConfigSpecific( mk, iconInstallationsBase, 'installation' ) );
        }
      }
    }
  }

  function zoomToBounds( map, bounds, maxZoom ) {
    if ( maxZoom ) {
      google.maps.event.addListenerOnce( map, 'bounds_changed', function() {
        if ( this.getZoom() > maxZoom ) {
          this.setZoom( maxZoom );
        }
      } );
    }

    map.fitBounds( bounds );
  }

} )( this );
