/**
 * SERVICE: LngMapService
 * @author Yoann
 */

( function( global ) {
  'use strict';

  angular
    .module( 'lng' )
    .factory( 'LngMapService', LngMapService );

  function LngMapService(
    $http,
    $state,
    $rootScope,
    $q,
    $filter,
    PhysicalQuantityService
  ) {

    var deferred,
      MapService,
      factory,
      script;

    // Google Maps Async loading
    deferred = $q.defer();

    window.googleMapsInitialized = function() {
      deferred.resolve( window.google.maps );
    };

    script = document.createElement( 'script' );
    script.type = 'text/javascript';
    script.src = 'https://maps.google.com/maps/api/js?v=3.34&key=AIzaSyDUsl_DM6LfVNqFSk7CvQXxBm2gqQ1gcHU&callback=googleMapsInitialized';
    document.body.appendChild( script );

    MapService = deferred.promise;

    factory = {
      createMap: createMap,
      addTrackToVessel: addTrackToVessel,
      addClickListener: addClickListener,
      addInstallations: addInstallations,
      addVessels: addVessels,
      addVoyageTrack: addVoyageTrack,
      setAutoZoom: setAutoZoom
    };

    return factory;

    ////////////

    function createMap( domElement, latitude, longitude ) {
      return MapService
        .then( createMapAndCenter )
        .then( waitTilesLoaded );


      ////////////

      function createMapAndCenter( gm ) {
        var center = new gm.LatLng(
          latitude,
          longitude
        );

        return global.mapUtils.createMap(
          gm,
          domElement,
          center );
      }

      function waitTilesLoaded( map ) {
        return $q( function( resolve ) {
          var listener = google.maps.event.addListener( map, 'tilesloaded', function() {
            google.maps.event.removeListener( listener );

            resolve( map );
          } );
        } );
      }
    }

    function addClickListener( mapPromise, callback ) {
      return ready( mapPromise )
        .then( function( map ) {
          map.$oms.addListener( 'click', callback );
          google.maps.event.addListener( map, 'click', callback );
        } );
    }

    function addTrackToVessel( mapPromise, vessel, trackPositions, zoom ) {
      return ready( mapPromise )
        .then( function( map ) {

          var mapProps = vessel.$mapAttrs,
            i = 0,
            len = trackPositions.length,
            position,
            mapPosAttrs,
            positions;

          // Populate positions
          for ( i; i < len; i++ ) {
            position = trackPositions[ i ];

            mapPosAttrs = {
              $pos: new google.maps.LatLng( position.$geo ),
              $loaded: position.currentCargo.loaded,
              $isCurrentCargo: global.displayHelper.isCurrentCargoTracked( position.currentCargo ),
              $angle: position.course || 0,
              $icon: global.clazzHelper.getItemClazz( position.type ),
              $name: vessel.name
            };
            position.$mapAttrs = mapPosAttrs;

          }

          // Add track & position marker
          mapProps.$track = global.markerVesselTrackHelper.create(
            map,
            trackPositions,
            mapProps.$loaded
          );

          // Zoom
          positions = _.map( trackPositions, '$geo' );

          if ( zoom ) {
            zoomToMap( map, positions, 9 );
          }

          return mapProps.$track;
        } );
    }

    function addVessels( map, vessels ) {
      var i = 0,
        len = vessels.length,
        filterTranslate = $filter( 'translate' ),
        // Create an object gathering properties that will common to each markers
        // This object is extended with vessel contextualized info
        zoom = map.getZoom(),
        iconConfig = global.markerIconHelper.iconConfigShared( zoom, 'vessel' );

      for ( i; i < len; i++ ) {
        add( vessels[ i ] );
      }

      ////////////

      function add( vessel ) {
        var mapProps,
          marker;

        if ( vessel.position ) {

          populateVessel();
          mapProps = vessel.$mapAttrs;

          if ( !mapProps.$marker ) {

            // Create marker
            marker = global.markerVesselHelper.create( map, vessel, iconConfig, filterTranslate );

            if ( mapProps.$marker ) {
              marker.$label = mapProps.$marker.$label;
            }

            // Bind click event on tooltip
            marker.$tooltip.addClickListener( function( ev ) {

              // stopPropagation required otherwise
              // event propagates to the map (or any element below the clicked position) and modify $stateParams
              ev.preventDefault();
              ev.stopPropagation();

              $rootScope.$apply( function() {
                $state.go( 'session.vessel', {
                  itemId: marker.$itemId
                } );
              } );
            } );


            // Remove old marker & old tooltip
            global.markerVesselHelper.remove( map, vessel );

            // Save new marker
            mapProps.$marker = marker;
          }

          mapProps.$marker.setMap( map );
        }


        ////////////

        function populateVessel() {

          var mapAttrs = {};

          if ( vessel.$mapAttrs ) {
            return;
          }

          if ( vessel.position ) {
            mapAttrs = {
              $pos: new google.maps.LatLng( vessel.$geo ),
              $volume: PhysicalQuantityService.format( vessel.position.quantity ),
              $loaded: vessel.position.currentCargo.loaded,
              $isCurrentCargo: global.displayHelper.isCurrentCargoTracked( vessel.position.currentCargo ),
              $angle: vessel.position.course || 0,
              $speed: vessel.position.speed || 0,
              $opacity: getVesselOpacity()
            };
          }

          mapAttrs.$icon = global.clazzHelper.getItemClazz( vessel.type );

          vessel.$mapAttrs = mapAttrs;


          ////////////

          function getVesselOpacity() {
            var receivedTime = vessel.position.$receivedTime;

            if ( receivedTime > moment().subtract( 1, 'day' ) ) {
              return 1;
            }

            if ( receivedTime > moment().subtract( 5, 'days' ) ) {
              return 0.8;
            }

            if ( receivedTime > moment().subtract( 1, 'month' ) ) {
              return 0.6;
            }

            return 0.4;
          }

        }
      }
    }

    function addInstallations( map, installations ) {

      var i = 0,
        len = installations.length,

        // Create an object gathering properties that will common to each markers
        // This object is extended with vessel contextualized info
        zoom = map.getZoom(),
        iconConfig = global.markerIconHelper.iconConfigShared( zoom, 'installation' );

      for ( i; i < len; i++ ) {
        add( installations[ i ] );
      }

      ////////////

      function add( installation ) {
        var mapProps,
          marker;

        if ( installation.$geo ) {

          mapProps = installation.$mapAttrs || {
            $pos: new google.maps.LatLng( installation.$geo )
          };

          if ( !mapProps.$marker ) {
            installation.$mapAttrs = mapProps;

            // create new marker
            marker = global.markerInstallationHelper.create( map, installation, iconConfig );

            // remove old marker
            global.markerInstallationHelper.remove( map, installation );

            // Save new marker
            mapProps.$marker = marker;
          }

          mapProps.$marker.setMap( map );
        }
      }
    }

    function addVoyageTrack( mapPromise, waypoints ) {
      return ready( mapPromise )
        .then( function( map ) {

          var track = global.markerVoyageTrackHelper.create( map, waypoints );

          zoomToMap( map, waypoints, 9 );

          return track;
        } );
    }

    function ready( mapPromise ) {
      return $q( function( resolve, reject ) {
        $q.all( [ MapService, mapPromise ] )
          .then( function( promises ) {
            var map = promises[ 1 ];
            return resolve( map );
          }, reject );
      } );
    }

    function setAutoZoom( mapPromise ) {
      return ready( mapPromise )
        .then( function( map ) {
          map.setOptions( {
            mapTypeControl: false
          } );

          return map;
        } );
    }

    /**
     * Zoom on a map viewport defined with an Array of Google Maps compliant coordinates
     * {lat: number, lng: number}
     * @param  {GoogleMapsObject} map -> Google Maps instance
     * @param  {[{lat:<number>, lng:<number>}]} positions
     * @param  {Number} maxZoom
     */
    function zoomToMap( map, positions, maxZoom ) {

      var bounds = new google.maps.LatLngBounds();

      if ( !positions || !positions.length ) {
        return;
      }

      _.each( positions, function( position ) {
        bounds.extend( new google.maps.LatLng( position ) );
      } );

      global.mapUtils.zoomToBounds( map, bounds, maxZoom );

    }
  }

} )( this );
