/**
 * SERVICE: MapCacheService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'MapCacheService', MapCacheService );

  function MapCacheService( $q, $rootScope, $filter, MapApiTransformsService, Restangular ) {

    var promise,
      vesselsCached = {},
      installationsCached = {},
      fleetMetrics = {},
      filterTranslate = $filter( 'translate' ),

      factory = {
        load: load,
        getMaxCapacity: getMaxCapacity,
        getCachedVesselById: getCachedVesselById,
        getCachedInstallationById: getCachedInstallationById,
        getVesselById: getVesselById,
        getInstallationById: getInstallationById,
        getFleetMetrics: getFleetMetrics
      };

    return factory;


    ////////////

    function extractData( res ) {
      return res.plain();
    }

    function load() {

      var loadVessels,
        loadInstallations,
        loadVesselMetrics;

      if ( !promise ) {

        loadVessels = Restangular.one( 'vessels' ).get().then( extractData ).then( updateAndStoreVessels );
        loadInstallations = Restangular.one( 'installations' ).get().then( extractData ).then( updateAndStoreInstallations );
        loadVesselMetrics = Restangular.one( 'vessels/metrics' ).get().then( extractData ).then( storeFleetMetrics );

        promise = $q.all( [
          loadVessels,
          loadInstallations,
          loadVesselMetrics
        ] ).catch( cleanPromise );
      }

      return promise;
    }

    function getVesselById( id ) {
      return load().then( function() {
        return vesselsCached[ id ];
      } );
    }

    function getInstallationById( id ) {
      return load().then( function() {
        return installationsCached[ id ];
      } );
    }

    function getCachedVesselById( id ) {
      return vesselsCached[ id ];
    }

    function getCachedInstallationById( id ) {
      return installationsCached[ id ];
    }

    function getFleetMetrics() {
      return fleetMetrics;
    }

    function getMaxCapacity( vessel ) {
      return _.get( vessel, 'capacityMultiUnits' ) || fleetMetrics.cargoQuantity.max;
    }

    function updateAndStoreVessels( vessels ) {

      var i = 0,
        vessel,
        vesselsLength;

      if ( vessels ) {

        vesselsLength = vessels.length;

        for ( i; i < vesselsLength; i++ ) {
          vessel = vessels[ i ];

          // Various formatting action
          // => Should move these methods to another service
          updateVesselLastPosition( vessel );
          updateVesselNextDestination( vessel );
          updateVesselController( vessel );
          updateItem( vessel );

          vessel.$geo = MapApiTransformsService.formatPosition( vessel.position );

          // Cache Vessel
          if ( vesselsCached[ vessel.id ] ) {
            vessel.$mapAttrs = vesselsCached[ vessel.id ].$mapAttrs;
          }
          vesselsCached[ vessel.id ] = vessel;

        }
      } else {
        showError();
      }
      return vessels;
    }

    function updateAndStoreInstallations( installations ) {

      var i = 0,
        installation,
        installationsLength;

      if ( installations ) {

        installationsLength = installations.length;

        for ( i; i < installationsLength; i++ ) {
          installation = installations[ i ];

          updateItem( installation );

          installation.$geo = MapApiTransformsService.formatPosition( installation.position );
          installation.type = MapApiTransformsService.formatInstallationType( installation.type );

          installationsCached[ installation.id ] = installation;

        }
      }
      return installations;
    }

    function storeFleetMetrics( data ) {
      fleetMetrics = data;
      return data;
    }

    function cleanPromise() {
      promise = null;
    }

    function updateItem( item ) {
      // homogeneous structure with installations
      if ( !item.type ) {
        if ( item.resourceType === 'vessel' ) {
          item.type = 'Vessel';
        } else {
          item.type = 'Unknown';
        }
      }

      // preprocess data for display
      item.name = item.name ? item.name : ( item.status === 'Under Construction' ? 'Vessel under construction' : 'Unnamed' );
      item.isMoving = item.position && item.position.speed ? ( item.position.speed >= 1 ) : false;
    }

    function updateVesselNextDestination( vessel ) {
      if ( vessel.nextDestination ) {
        vessel.$nextDestinationPlace = vessel.nextDestination.installation || vessel.nextDestination.zone;

        if ( vessel.nextDestination.eta ) {
          vessel.$nextDestinationEta = global.datesUtils.momentFullTime( vessel.nextDestination.eta );
        }
      }

      return vessel;
    }

    function updateVesselController( vessel ) {
      var organisationController;
      if ( vessel.vesselController ) {
        organisationController = _( vessel.vesselController ).keys().pull( 'default' ).value();
        //case when we have a specific organisation vessel controller

        if ( organisationController.length > 0 ) {
          vessel.$vesselController = vessel.vesselController[ organisationController[ 0 ] ];
        } else {
          //default controller
          vessel.$vesselController = vessel.vesselController.default;
        }
      }

      return vessel;
    }

    function updateVesselLastPosition( item ) {
      if ( item.lastPosition ) {
        item.position = item.lastPosition;
        item.position.$receivedTime = moment( item.position.receivedTimeUtc || item.position.receivedTime );
        item.position.$receivedTimeHuman = global.datesUtils.humanize( item.position.receivedTimeUtc || item.position.receivedTime );
        delete item.lastPosition;
      }
    }


    function showError() {
      $rootScope.$emit( 'toast:error', filterTranslate( 'TOAST.DATA_RELOAD_FAILED' ) );
    }
  }

} )( this );
