/**
 * Marker Icon Helper
 * This file contains methods used to optimize the preparation and manipulation
 * icons linked to each markers of the map.
 */

( function( global ) {
  'use strict';

  global.markerIconHelper = ( global.module || {} ).exports = {
    iconConfigShared: iconConfigShared,
    iconConfigSelected: iconConfigSelected,
    iconConfigSpecific: iconConfigSpecific
  };

  /**
   * @description:
   *
   * A method to process icon properties SHARED by each markers
   *
   * These properties depends on the zoom level and resource type
   *
   * This methods must be used outside of for loop processing markers so that
   * we can avoid creating unneeded instances of google.maps.Point and google.maps.Size
   * It is necessary to use the `iconConfigSpecific` method inside for loop` processing markers
   * to get the complete icon configuration.
   *
   * @param {int} zoom - Current google map zoom level
   * @param {string} type - Resource type associated to the icon
   * @returns {{anchor: google.maps.Point, size: google.maps.Size, url: string}} - List of icon properties shared by each markers
   */
  function iconConfigShared( zoom, type ) {

    var markerSize,
      url,
      anchor,
      scale = ( zoom > 3 ) ? ( zoom > 7 ? 1 : 0.75 ) : 0.5,
      spriteWidth,
      spriteHeight;

    if ( type === 'vessel' ) {
      spriteWidth = 1080;
      spriteHeight = 600;
      markerSize = 30 * scale;
      url = './img/vessels-sprite.png';
      anchor = new google.maps.Point( markerSize / 2, markerSize / 2 );
    } else if ( type === 'installation' ) {
      spriteWidth = 30;
      spriteHeight = 1200;
      markerSize = 30 * scale;
      url = './img/installations-sprite.png';
      anchor = new google.maps.Point( markerSize / 3, markerSize );
    }

    return {
      anchor: anchor,
      size: new google.maps.Size( markerSize, markerSize ),
      url: url,
      scaledSize: new google.maps.Size( scale * spriteWidth, scale * spriteHeight ),
      // Not required by Google API, required to calculate 'origin' property in markerIconSpecific
      $markerSizeX: markerSize,
      $markerSizeY: markerSize
    };
  }

  /**
   * @description:
   *
   * A method to process icon properties SPECIFIC to each vessel markers
   *
   * This method extends object returned by markerIconShared method and markerIconSelected method
   *
   * @param {object} marker
   * @param {object} config returned by iconConfigSgared method
   * @param {string} ['vessel'] - resource type
   * @returns {{anchor: google.maps.Point, size: google.maps.Size, url: string}} : List of icon properties shared by vessel markers
   */
  function iconConfigSpecific( marker, iconConfig, type ) {

    var spritePositionX = 0,
      spritePositionY = 0;

    type = type || 'vessel';

    if ( type === 'vessel' ) {
      spritePositionX = Math.floor( ( ( marker.$angle + 360 ) % 360 ) / 10 );
      spritePositionY = vesselIconSpriteCoordinate( marker );
    } else if ( type === 'installation' ) {
      spritePositionY = installationIconSpriteCoordinate( marker.type || marker.$type );
    }

    // Extends iconConfig to match Google API requirements
    // to create an icon
    iconConfig.origin = new google.maps.Point(
      iconConfig.$markerSizeX * spritePositionX,
      iconConfig.$markerSizeY * spritePositionY
    );

    return iconConfig;
  }

  /**
   * @description:
   * A method to process icon properties of selected vessel
   * This is not needed for installations
   *
   * @returns {{anchor: google.maps.Point, size: google.maps.Size, url: string}}
   */
  function iconConfigSelected() {
    var scale = 1.25,
      spriteWidth = 1080,
      spriteHeight = 600,
      markerSize = 30 * scale;

    return {
      anchor: new google.maps.Point( markerSize / 2, markerSize / 2 ),
      size: new google.maps.Size( markerSize, markerSize ),
      url: './img/vessels-sprite.png',
      scaledSize: new google.maps.Size( scale * spriteWidth, scale * spriteHeight ),
      // Not required by Google API, required to calculate 'origin' property in markerIconSpecific
      $markerSizeX: markerSize,
      $markerSizeY: markerSize
    };
  }


  /**
   * Map Installation type with a int value to be able to
   * get correct icon position in the sprite image
   * @param itemType
   * @returns {number}
   */
  function installationIconSpriteCoordinate( itemType ) {
    switch ( global.installationHelper.getMainType( itemType ) ) {
      case 'Export':
        return 0;
      case 'Import':
        return 10;
      case 'Shipyard':
        return 20;
      case 'Anchorage':
        return 30;
      default:
        return -1;
    }
  }

  /**
   * Map Installation type with a int value to be able to
   * get correct icon position in the sprite image
   * @param {object} Vessel Marker
   * @returns {number}
   */
  function vesselIconSpriteCoordinate( marker ) {
    var x,
      y,
      SPRITE_CONFIG = {
        loaded: 0,
        loadedHover: 1,
        ballast: 2,
        ballastHover: 3,
        open: 4,
        storage: 5,
        underConstruction: 6,
        laidUp: 7,
        otherCommo: 8,
        otherCommoHover: 9
      },
      N_ICONS_VARIATIONS = _.uniq( _.values( SPRITE_CONFIG ) ).length;

    x = _.get( marker, '$speed', 0 ) > 1 ? 0 : 1;
    switch ( marker.$state ) {
      case 'open':
        y = SPRITE_CONFIG.open;
        break;
      case 'underConstruction':
        y = SPRITE_CONFIG.underConstruction;
        break;
      case 'laidUp':
        y = SPRITE_CONFIG.laidUp;
        break;
      case 'Floating storage':
        y = SPRITE_CONFIG.storage;
        break;
      default:
        if ( marker.$loaded ) {
          if ( marker.$isCurrentCargo ) {
            y = marker.$selected ? SPRITE_CONFIG.loadedHover : SPRITE_CONFIG.loaded;
          } else {
            y = marker.$selected ? SPRITE_CONFIG.otherCommoHover : SPRITE_CONFIG.otherCommo;
          }
        } else {
          y = marker.$selected ? SPRITE_CONFIG.ballastHover : SPRITE_CONFIG.ballast;
        }
        break;
    }

    return N_ICONS_VARIATIONS * x + y;
  }


} )( this );
