/**
 * Map Helper
 * @author Yoann
 */

( function( global ) {
  'use strict';

  global.markerVesselHelper = ( global.module || {} ).exports = {
    create: create,
    remove: remove,
    select: select,
    unselect: unselect
  };

  ////////////

  /**
   * Create a new Marker for each vessel
   * This method is only called at application start once for each vessel
   *
   * @param map - Active Google Map instance
   * @param vessel - A vessel object returned by Kpler API
   * @param iconSharedConfig - List of icon properties shared by each vessel markers
   * @param filterTranslate - Angular Filter handling text translation
   * @returns {Marker}
   */
  function create( map, vessel, iconSharedConfig, filterTranslate ) {

    var mapAttrs = vessel.$mapAttrs,
      icon = global.markerIconHelper.iconConfigSpecific( mapAttrs, iconSharedConfig ),
      marker = new google.maps.Marker( {
        position: mapAttrs.$pos,
        map: map,
        icon: icon,
        zIndex: google.maps.Marker.MAX_ZINDEX,
        opacity: mapAttrs.$opacity
      } );

    marker.$remove = removeMarker;
    // Dirty. Cache some data to access them in lng-map.helper.js
    marker.$itemType = 'vessel';
    marker.$itemId = vessel.id;
    marker.$speed = mapAttrs.$speed;
    marker.$angle = mapAttrs.$angle;
    marker.$loaded = mapAttrs.$loaded;
    marker.$isCurrentCargo = mapAttrs.$isCurrentCargo;
    map.$oms.addMarker( marker );

    addTooltip();

    return marker;

    ////////////

    function removeMarker() {
      this.setMap( null );

      if ( this.$tooltip ) {
        this.$tooltip.setMap( null );
      }
    }

    function addTooltip() {

      var className,
        content = createVesselTootipContent( mapAttrs.$volume );

      if ( mapAttrs.$loaded ) {
        className = mapAttrs.$isCurrentCargo ? 'map-tooltip-loaded' : 'map-tooltip-other';
      } else {
        className = 'map-tooltip-ballast';
      }

      marker.$tooltip = global.newMapTooltip( google.maps, {
        marker: marker,
        className: className,
        content: content
      } );

      ////////////

      function createVesselTootipContent( volume ) {
        var translateLoaded = 'Loaded',
          translateBallast = 'Ballast',
          translateProduct = 'Other',
          translateSpeed = 'Speed',
          translateDraught = 'draught',
          translatePosition = 'Position',
          translateETA = 'ETA',
          carriedCargoText;

        if ( filterTranslate ) {
          translateLoaded = filterTranslate( 'COMPONENTS.MARKERS.LOADED' );
          translateBallast = filterTranslate( 'COMPONENTS.MARKERS.BALLAST' );
          translateSpeed = filterTranslate( 'COMPONENTS.MARKERS.SPEED' );
          translateDraught = filterTranslate( 'COMPONENTS.MARKERS.DRAUGHT' );
          translatePosition = filterTranslate( 'COMPONENTS.MARKERS.POSITION' );
          translateETA = filterTranslate( 'COMPONENTS.MARKERS.ETA' );
        }

        if ( mapAttrs.$loaded ) {
          carriedCargoText = mapAttrs.$isCurrentCargo ? translateLoaded + ' (' + volume + ')' : translateProduct;
        } else {
          carriedCargoText = translateBallast;
        }

        return '<div class="map-tooltip-btn"><button type="button"><i class="fa fa-chevron-right"></i></button></div>' +
          '<table>' +
          '<tr><td><i class="fa ' + mapAttrs.$icon + '"></i>&nbsp;</td><td><div class="map-tooltip-name">' + vessel.name + '</div></td></tr>' +
          '<tr><td></td><td><i class="fa fa-adjust"></i> ' + carriedCargoText + '</td></tr>' +
          ( vessel.$vesselController ? '<tr><td></td><td><i class="fa fa-lng-player"></i> ' + vessel.$vesselController.name + '</td></tr>' : '' ) +
          ( vessel.$nextDestinationPlace ? '<tr><td></td><td><i class="fa fa-arrow-right"></i> ' + vessel.$nextDestinationPlace.name + '</td></tr>' : '' ) +
          ( vessel.$nextDestinationEta ? '<tr><td></td><td>' + translateETA + ' ' + vessel.$nextDestinationEta + '</td></tr>' : '' ) +
          '<tr ><td></td><td style="padding-top: 5px;"><div style="padding-top: 5px; border-top: 1px solid white;"> <i class="fa fa-lng-raw-portcall"></i> ' + translatePosition + '</div></td></tr>' +
          '<tr><td></td><td style="padding-left: 14px;">' + ( vessel.resourceType === 'vessel' ? vessel.position.$receivedTimeHuman : '{{vessel.$statusText | translate}}' ) + '</td></tr>' +
          ( vessel.resourceType === 'vessel' ? (
            ( vessel.position.draught ? '<tr><td></td><td style="padding-left: 14px;"> ' + translateDraught + ': ' + vessel.position.draught + ' m </td></tr>' : '' ) +
            '<tr><td></td><td style="padding-left: 14px;">' + translateSpeed + ': ' + vessel.position.speed + ' kn </td></tr>'
          ) : '' ) +
          '</table>';
      }
    }
  }

  function remove( map, item ) {
    var mapAttrs,
      marker;

    if ( item ) {
      mapAttrs = item.$mapAttrs;

      // remove marker
      marker = mapAttrs.$marker;
      if ( marker ) {
        marker.setMap( null );
        map.$oms.removeMarker( marker );
      }
    }
  }

  function select( map, marker ) {
    var icon = global.markerIconHelper.iconConfigSelected();

    marker.$tooltip.setMap( map );
    marker.$selected = true;
    icon = global.markerIconHelper.iconConfigSpecific( marker, icon );
    marker.setIcon( icon );

  }

  function unselect( map, marker ) {
    var icon = global.markerIconHelper.iconConfigShared( map.getZoom(), 'vessel' );

    marker.$tooltip.setMap( null );
    marker.$selected = false;
    icon = global.markerIconHelper.iconConfigSpecific( marker, icon );
    marker.setIcon( icon );

  }

} )( this );
