/**
 * PhysicalQuantityService
 * @todo: Improve scale option
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'PhysicalQuantityService', PhysicalQuantityService );

  function PhysicalQuantityService( $filter, $parse, UnitsConfig ) {

    var PRECISION_FOR_ABS_LOWER_THAN_UNIT = 1,
      factory;

    factory = {
      legend: legend,
      format: format,
      getValue: getValue,
    };

    return factory;

    //////////// Public methods

    /**
     * Get unit symbol
     * This is useful to legend table-columns containing unitless values
     * @param {object} options
     * @param {boolean} [options.useAggregated = false] Use unit configured by 'Aggregated' of 'Cargo' setting
     * @param {string}  [options.scale=''] Add a math indication before the unit if the displayed value described by the legend has been scaled
     *                                     Accepts a unit prefix among [k, M, G]
     *                                     For instance:
     *                                       - original value is 143000 and unit is m3
     *                                       - We want to display 143 as a value and (x1000) m3 as a legend
     *                                       - Using options.scale = 'k' allows to get this legend
     * @param {boolean} [options.perYear = false] Add 'pa' suffix to the unit if set to 'true'
     * @returns {string} - formatted legend (can contain HTML tags) : m3; (x1000) m3; mtpa
     */
    function legend( options ) {

      var unitObject,
        useAggregated,
        perYear,
        scale,
        symbol;

      // Set default options
      useAggregated = ( typeof options.useAggregated === 'boolean' ) ? options.useAggregated : false;
      scale = ( [ 'k', 'M', 'G' ].indexOf( options.scale ) > -1 ) ? options.scale : '';
      perYear = ( typeof options.perYear === 'boolean' ) ? options.perYear : false;

      // Find complete unit object associated to the unit chosen by the user
      unitObject = getUnitObject( useAggregated );

      symbol = unitObject.symbol;

      // Append 'per annum' suffix
      if ( perYear ) {
        symbol += 'pa';
      }

      // Check if scale indication needs to be applied
      // Add (x1000) after the unit for instance if scale: 'k' is used in options
      if ( scale.length ) {
        symbol += ' (x' + convertPrefix( scale ) + ')';
      }

      return symbol;

    }

    /**
     * Format raw data provided by the API to be displayed with the unit specified by settings.displayUnit.cargo configuration
     *
     * @param {Object} physicalQuantity          Physical value converted in mass, volume and energy (sent by the API)
     * @param {number} physicalQuantity.mass        Mass dimension of the physical value in Million Tons (Mt)
     * @param {number} physicalQuantity.volume      Volume dimension of the physical value in Cubic meters (m3)
     * @param {number} physicalQuantity.energy      Energy dimension of the physical value in ??? (???) @todo: do not know yet if energy will be returned as kWh or MMBtu
     * @param {boolean} physicalQuantity.per_year   add per annum unit if true (Mtpa for instance)
     *
     * @param {Object} options
     * @param {boolean} [options.showUnit = true] Specify if the physical quantity should be rendered with or without unit.
     *                                           If set to false, a legend should be used (@see: legend method)
     *
     * @param {boolean} [options.useAggregated = false] Use unit configured by 'Aggregated' of 'Cargo' setting
     *
     * @param {number} [options.precision] Force the precision of the display
     *
     * @param {string} [options.scale=''] Accepts a unit prefix among [k, M, G]. Will divide original value and display the prefix
     *
     * @returns {string} formatted number value + selected unit
     */
    function format( physicalQuantity, options ) {

      var computedValue,
        precision,
        formattedValue,
        scale,
        showUnit,
        symbol,
        unitObject,
        useAggregated;

      if ( !physicalQuantity ) {
        return;
      }

      if ( _.isObject( physicalQuantity ) && ( !physicalQuantity.volume && !physicalQuantity.mass && !physicalQuantity.energy ) ) {
        return;
      }

      options = options || {};

      // Defaults values
      symbol = '';
      showUnit = ( typeof options.showUnit === 'boolean' ) ? options.showUnit : true;
      precision = ( typeof options.precision === 'number' ) ? options.precision : 0;
      scale = ( [ 'k', 'M', 'G' ].indexOf( options.scale ) > -1 ) ? options.scale : '';
      useAggregated = ( typeof options.useAggregated === 'boolean' ) ? options.useAggregated : false;

      unitObject = getUnitObject( useAggregated );
      computedValue = getComputedValue( physicalQuantity, unitObject );

      // Check if there is a computed value (sometimes API does not return data for a given dimension...)
      if ( computedValue ) {

        // Check if unit is asked for display
        if ( showUnit ) {
          // Add 'pa' suffix (per annum) to the unit if necessary
          // @todo: Maybe irrelevant to use this for some units ? If so, limit possible unit conversions
          symbol = unitObject.symbol;
          if ( physicalQuantity.perYear ) {
            symbol += 'pa';
          }
        } else if ( scale ) {
          // Scale the numerical value
          computedValue = computedValue / convertPrefix( scale );
        }

        // Format the number and add the unit if existing
        if ( Math.abs( computedValue ) >= 100 ) {
          formattedValue = $filter( 'number' )( computedValue, 0 );
        } else if ( Math.abs( computedValue ) >= 1 ) {
          formattedValue = $filter( 'number' )( computedValue, precision );
        } else {
          formattedValue = parseFloat( computedValue.toPrecision( PRECISION_FOR_ABS_LOWER_THAN_UNIT ) );
        }
        formattedValue += ' ' + symbol;
        formattedValue.trim();

        return formattedValue;

      } else {
        console.warn( 'Can\'t render physical data - The ' + unitObject.dimension + ' dimension contains null value' );
        return;
      }

    }

    /**
     * Get the physicalQuantity value expressed in the current active unit.
     * @param {object} physicalQuantity - Physical value converted in mass, volume and energy (sent by the API) (@see format method)
     *
     * @param {object} options
     * @param {boolean} [options.useAggregated = false] Use unit configured by 'Aggregated' of 'Cargo' setting
     *
     * @returns {number} The physical quantity value expressed in the unit active when the method is called
     */
    function getValue( physicalQuantity, options ) {
      var computedValue,
        formattedValue,
        unitObject,
        useAggregated;

      options = options || {};

      if ( typeof physicalQuantity === 'number' ) {
        return physicalQuantity;
      }

      if ( !physicalQuantity ) {
        return;
      }

      if ( _.isObject( physicalQuantity ) && ( !physicalQuantity.volume && !physicalQuantity.mass && !physicalQuantity.energy ) ) {
        return;
      }

      // Defaults values
      useAggregated = ( typeof options.useAggregated === 'boolean' ) ? options.useAggregated : false;

      unitObject = getUnitObject( useAggregated );
      computedValue = getComputedValue( physicalQuantity, unitObject );

      if ( computedValue ) {
        formattedValue = ( Math.abs( computedValue ) >= 1 ) ? computedValue : parseFloat( computedValue.toPrecision( PRECISION_FOR_ABS_LOWER_THAN_UNIT ) );
      } else {
        console.warn( 'Can\'t render physical data - The ' + unitObject.dimension + ' dimension contains null value' );
        return;
      }


      return formattedValue;
    }

    /**
     * Looks for and returns the unit object with the id specified as the argument
     * @param {string} - id of the unit to retrieve from available units
     * @returns {object} - Unit object
     * @see units.config.js
     */
    function getUnitObject( useAggregated ) {

      // Load units configuration
      var currentCommo = JSON.parse( localStorage.getItem( 'current_commodity' ) ),
        id = useAggregated ? UnitsConfig.default[ currentCommo ].aggregated : UnitsConfig.default[ currentCommo ].cargo,
        units = JSON.parse( localStorage.getItem( 'availableUnits' ) );

      return _.find( units, function( unitObject ) {
        return unitObject.name === id;
      } );
    }

    //////////// Private methods

    /**
     *
     */
    function getComputedValue( physicalQuantity, unitObject ) {

      var physicalDimension,
        originalValue,
        computedValue,
        formula;

      // Get data dimension to know what value to use from data exposed by the API
      physicalDimension = unitObject.dimension;

      if ( _.isObject( physicalQuantity ) ) {
        originalValue = physicalQuantity[ physicalDimension ];
      } else {
        // @todo: ensure compatibility for data that is not yet moved to the physical-data object representation. Remove this in the future
        originalValue = physicalQuantity;
      }

      if ( originalValue ) {
        // Compute Physical Value to display according to current unit
        // Parse and evaluate unit conversion formula to get computed value to display
        formula = $parse( unitObject.conversionFormula );
        computedValue = formula( { x: originalValue } );
      } else {
        computedValue = '';
      }

      return computedValue;

    }

    /**
     * Convert a unit prefix to its corresponding power of ten
     * @param {string} prefix
     * @param {string} [returnedType = 'number'] - Specify if the result should be a number (to perform calculations) or a string (to be used for legend description)
     * @returns {string|number} Power of Ten corresponding to the prefix
     */
    function convertPrefix( prefix, returnedType ) {
      var result;

      // Default optional value
      returnedType = returnedType || 'number';

      //
      switch ( prefix ) {

        case 'k':
          result = 1000;
          break;
        case 'M':
          result = 1e6;
          break;
        case 'G':
          result = 1e9;
          break;
        default:
          result = ( returnedType === 'number' ) ? 1 : '';

      }

      return result;
    }

  }

} )();
