/**
 * DIRECTIVE: trade
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'trade', trade );

  function trade() {
    var directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        trade: '='
      },
      templateUrl: 'components/trades/trade/trade.html',
      controller: TradeController,
      controllerAs: 'vm'
    };

    return directive;


    ////////////

    /* @ngInject */
    function TradeController(
      $state,
      MapCacheService,
      PhysicalQuantityService
    ) {

      var vm = this,
        defaultDate = global.datesUtils.momentFullDate,
        quantity = this.trade.flowQuantityFromOrigin || this.trade.flowQuantityFromDestination;

      // Expose state context to the template so that we can adapt block to display
      vm.context = $state.current.name;
      vm.trade.$flowQuantity = PhysicalQuantityService.getValue( quantity );

      if ( vm.trade.vessels && vm.trade.vessels.length > 0 ) {
        vm.trade.$vessel = vm.trade.vessels.reduce( function( high, next ) {
          return next.capacity > high.capacity ? next : high;
        } );
        vm.trade.$vesselCapacity = _.flowRight(
            PhysicalQuantityService.getValue,
            MapCacheService.getMaxCapacity
        )( vm.trade.$vessel );
      }

      updateTrade();

      ////////////

      function updateTrade() {

        var p,
          exportTradeLink,
          importTradeLink;

        vm.tradeLinks = updateTradeLinkSequences();
        vm.prices = updatePrices();

        p = vm.prices;

        if ( vm.tradeLinks && vm.tradeLinks.length ) {
          //fisrt TL is the export one
          exportTradeLink = vm.tradeLinks[ 0 ];
          //last TL is the import one
          importTradeLink = vm.tradeLinks[ vm.tradeLinks.length - 1 ];

          if ( p ) {
            if ( p.export ) {
              exportTradeLink.$exportPrice = {
                value: p.export.averagePrices.toFixed( 2 ),
                number: p.export.nbAggregation
              };
            }

            if ( p.import ) {
              importTradeLink.$importPrice = {
                value: p.import.averagePrices.toFixed( 2 ),
                number: p.import.nbAggregation
              };
            }
          }
        }


        vm.origin = findTarget( vm.trade.portCallOrigin );
        vm.destination = findTarget( vm.trade.portCallDestination );

        if ( vm.trade.start ) {
          vm.startHuman = defaultDate( vm.trade.start );
        } else if ( vm.trade.estimatedStart ) {
          vm.startHuman = defaultDate( vm.trade.estimatedStart );
        }

        if ( vm.trade.end ) {
          vm.endHuman = defaultDate( vm.trade.end );
        } else if ( vm.trade.$estimatedEnd ) {
          vm.endHuman = defaultDate( vm.trade.estimatedEnd );
        }

        vm.clazz = global.clazzHelper.getTimeInfoClazz( vm.trade.start, vm.trade.end );

        ////////////

        function updateTradeLinkSequences() {
          var tradeLinkSequenceRaw,
            bestTradeLinkSequenceRaw,
            bestTradeLinkSequence,
            allTradeLinkSequence,
            tradeLinkRow,
            maxTradelinks,
            i;
          tradeLinkSequenceRaw = ( vm.trade.orgSpecificInfo && vm.trade.orgSpecificInfo.default && vm.trade.orgSpecificInfo.default.bestTradeLinkSequences ) || [];
          bestTradeLinkSequenceRaw = _.filter( tradeLinkSequenceRaw, { selected: true } );
          bestTradeLinkSequence = [];
          allTradeLinkSequence = [];
          maxTradelinks = 0;
          i = 0;

          if ( tradeLinkSequenceRaw && tradeLinkSequenceRaw.length > 0 ) {
            // A best tradelinksequence exists
            // we use it and format it to match template needs
            if ( bestTradeLinkSequenceRaw.length > 0 ) {
              maxTradelinks = bestTradeLinkSequenceRaw[ 0 ].tradeLinks.length;
              bestTradeLinkSequence = formatTradeLinkSequence( bestTradeLinkSequenceRaw, maxTradelinks );
              return bestTradeLinkSequence;
            } else {
              // No best options,
              // we display all the tradeLinkSequence options in a compact format
              for ( i = 0; i < bestTradeLinkSequenceRaw.length; i++ ) {
                maxTradelinks = Math.max( tradeLinkSequenceRaw[ i ].tradeLinks.length, maxTradelinks );
              }
              allTradeLinkSequence = formatTradeLinkSequence( tradeLinkSequenceRaw, maxTradelinks );

              return allTradeLinkSequence;
            }
          }

          ////////////

          /**
           * Format tradelinksequences (BEST or ALTERNATIVES) to match the template needs
           *  tradelink 1 : [type] [delivery] potential sellers --> potential buyers
           *  ...
           *  tradelink N : [type] [delivery] potential sellers --> potential buyers
           */
          function formatTradeLinkSequence( rawFormat, max ) {

            var tradeLinks = [],
              j;

            tradeLinks.length = max;

            // Build potential tradelinks array
            for ( i = 0; i < rawFormat.length; ++i ) {
              tradeLinkRow = rawFormat[ i ].tradeLinks;

              for ( j = 0; j < max; ++j ) {
                // Fix null seller / null buyer
                // until fixed in Scala side
                if ( !tradeLinkRow[ j ].seller ) {
                  tradeLinkRow[ j ].seller = 'N/A';
                }
                if ( !tradeLinkRow[ j ].buyer ) {
                  tradeLinkRow[ j ].buyer = 'N/A';
                }

                if ( !tradeLinks[ j ] ) {
                  tradeLinks[ j ] = {
                    'type': [ tradeLinkRow[ j ].type ],
                    'delivery': [ tradeLinkRow[ j ].delivery ],
                    'seller': [ tradeLinkRow[ j ].seller ],
                    'buyer': [ tradeLinkRow[ j ].buyer ],
                    'hasVesselSwap': rawFormat[ i ].vesselSwap
                  };
                } else {
                  tradeLinks[ j ].type.push( tradeLinkRow[ j ].type );
                  tradeLinks[ j ].delivery.push( tradeLinkRow[ j ].delivery );
                  tradeLinks[ j ].seller.push( tradeLinkRow[ j ].seller );
                  tradeLinks[ j ].buyer.push( tradeLinkRow[ j ].buyer );
                }
              }
            }

            // Remove duplicates
            for ( j = 0; j < max; ++j ) {
              tradeLinks[ j ].type = _.uniq( tradeLinks[ j ].type );
              tradeLinks[ j ].delivery = _.uniq( tradeLinks[ j ].delivery );
              tradeLinks[ j ].seller = _.uniq( tradeLinks[ j ].seller );
              tradeLinks[ j ].buyer = _.uniq( tradeLinks[ j ].buyer );
            }

            return tradeLinks;
          }
        }

        function updatePrices() {
          return vm.trade.orgSpecificInfo && vm.trade.orgSpecificInfo.default && vm.trade.orgSpecificInfo.default.prices;
        }

        function findTarget( pc ) {
          if ( !pc ) {
            return;
          }

          if ( pc.installation ) {
            return pc.installation;
          } else {
            return pc.zone;
          }
        }
      }

    }
  }

} )( this );
